package main

import (
	"git.gammaspectra.live/P2Pool/p2pool-observer/database"
	"git.gammaspectra.live/P2Pool/p2pool-observer/monero/address"
	"git.gammaspectra.live/P2Pool/p2pool-observer/p2pool/api"
	"golang.org/x/exp/maps"
	"golang.org/x/exp/slices"
)

// MapJSONBlock fills special values for any block
func MapJSONBlock(api *api.Api, block database.BlockInterface, extraUncleData, extraCoinbaseData bool) {

	b := block.GetBlock()
	b.Lock.Lock()
	defer b.Lock.Unlock()

	if extraCoinbaseData {

		var tx *database.CoinbaseTransaction
		if b.Main.Found {
			tx = api.GetDatabase().GetCoinbaseTransaction(b)
		}

		if b.Main.Found && tx != nil {
			b.Coinbase.Payouts = make([]*database.JSONCoinbaseOutput, len(tx.Outputs()))
			for _, output := range tx.Outputs() {
				miner := api.GetDatabase().GetMiner(output.Miner())
				b.Coinbase.Payouts[output.Index()] = &database.JSONCoinbaseOutput{
					Amount:  output.Amount(),
					Index:   output.Index(),
					Address: miner.Address(),
					Alias:   miner.Alias(),
				}
			}
		} else {
			payoutHint, _ := api.GetBlockWindowPayouts(b)

			addresses := make(map[address.PackedAddress]*database.JSONCoinbaseOutput, len(payoutHint))

			for minerId, amount := range payoutHint {
				miner := api.GetDatabase().GetMiner(minerId)
				addresses[*miner.MoneroAddress().ToPackedAddress()] = &database.JSONCoinbaseOutput{
					Address: miner.Address(),
					Alias:   miner.Alias(),
					Amount:  amount.Lo,
				}
			}

			sortedAddresses := maps.Keys(addresses)

			slices.SortFunc(sortedAddresses, func(a address.PackedAddress, b address.PackedAddress) bool {
				return a.Compare(&b) < 0
			})

			b.Coinbase.Payouts = make([]*database.JSONCoinbaseOutput, len(sortedAddresses))

			for i, key := range sortedAddresses {
				addresses[key].Index = uint64(i)
				b.Coinbase.Payouts[i] = addresses[key]
			}
		}
	}

	weight := b.Difficulty
	b.Uncles = make([]any, 0)

	if uncle, ok := block.(*database.UncleBlock); ok {
		b.Parent = &database.JSONBlockParent{
			Id:     uncle.ParentId,
			Height: uncle.ParentHeight,
		}

		weight = weight.Mul64(100 - api.GetP2PoolAPI().Consensus().UnclePenalty).Div64(100)
	} else {
		for u := range api.GetDatabase().GetUnclesByParentId(b.Id) {
			uncleWeight := u.Block.Difficulty.Mul64(api.GetP2PoolAPI().Consensus().UnclePenalty).Div64(100)
			weight = weight.Add(uncleWeight)

			if !extraUncleData {
				b.Uncles = append(b.Uncles, &database.JSONUncleBlockSimple{
					Id:     u.Block.Id,
					Height: u.Block.Height,
					Weight: u.Block.Difficulty.Mul64(100 - api.GetP2PoolAPI().Consensus().UnclePenalty).Div64(100).Lo,
				})
			} else {
				uncleMiner := api.GetDatabase().GetMiner(u.Block.MinerId)
				b.Uncles = append(b.Uncles, &database.JSONUncleBlockExtra{
					Id:         u.Block.Id,
					Height:     u.Block.Height,
					Difficulty: u.Block.Difficulty,
					Timestamp:  u.Block.Timestamp,
					Miner:      uncleMiner.Address(),
					MinerAlias: uncleMiner.Alias(),
					PowHash:    u.Block.PowHash,
					Weight:     u.Block.Difficulty.Mul64(100 - api.GetP2PoolAPI().Consensus().UnclePenalty).Div64(100).Lo,
				})
			}
		}
	}

	b.Weight = weight.Lo

	if b.IsProofHigherThanDifficulty() && !b.Main.Found {
		b.Main.Orphan = true
	}

	miner := api.GetDatabase().GetMiner(b.MinerId)
	b.Address = miner.Address()
	b.MinerAlias = miner.Alias()
}
